@@ -1,3 +1,4 @@
+# $Id: L10N.pm 478 2005-03-22 00:50:54Z claco $
 package Handel::L10N;
 use strict;
 use warnings;
@@ -15,9 +16,9 @@ BEGIN {
     _AUTO => 1
 );
 
-$handle = __PACKAGE__->get_handle();
-
 sub translate {
+    my $handle = __PACKAGE__->get_handle();
+
     return $handle->maketext(@_);
 };
 
@@ -28,10 +29,6 @@ __END__
 
 Handel::L10N - Localization module for Handel
 
-=head1 VERSION
-
-    $Id: L10N.pm 53 2004-12-31 23:13:51Z claco $
-
 =head1 SYNOPSIS
 
     use Handel::L10N qw(translate);
@@ -40,7 +37,7 @@ Handel::L10N - Localization module for Handel
 
 =head1 DESCRIPTION
 
-This module is simply a subclass of L<Localte::Maketext>. By default it doesn't
+This module is simply a subclass of C<Locale::Maketext>. By default it doesn't
 export anything. You can either use it directly:
 
     use Handel::L10N;
@@ -53,12 +50,12 @@ You can also export C<translate> into the users namespace:
 
     warn translate('My message');
 
-Thus far, the French translation comes from Googles translation tools.IF you
+Thus far, the French translation comes from Googles translation tools. IF you
 have the time and can do better, the help would be much appreciated.
 
 =head1 METHODS
 
-=head2 C<translate>
+=head2 translate
 
 Translates the supplied text into the appropriate language if available. If no
 match is available, the original text is returned.
@@ -71,9 +68,5 @@ L<Locale::Maketext>, L<Handel::L10N::us_en>, L<Handel::L10N::fr>
 
     Christopher H. Laco
     CPAN ID: CLACO
-    cpan@chrislaco.com
+    claco@chrislaco.com
     http://today.icantfocus.com/blog/
-
-
-
-
